package com.streambase.contrib;

import java.util.List;

import com.streambase.sb.CompleteDataType;
import com.streambase.sb.DataType;
import com.streambase.sb.NullValueException;
import com.streambase.sb.Schema;
import com.streambase.sb.Tuple;
import com.streambase.sb.TupleException;
import com.streambase.sb.Schema.Field;
import com.streambase.sb.client.CustomFunctionResolver;

public class BlackScholesInfo {

	/**
	 * 
	 * @param type the type of option it is
	 * @param info a list of prices and sensitivities	 * 
	 * @return a return tuple
	 * @throws TupleException 
	 * @throws NullValueException 
	 */
	@CustomFunctionResolver("blackScholesTupleCustomFunctionResolver")
	public static Tuple blackScholesTuple(String type, List<Double> info) throws NullValueException, TupleException {
		if (type == null || info == null || info.size() != 7)
			return null;
		Field[] f = new Field[8];
		
		// Set Fields
		f[0] = Schema.createField(DataType.STRING, "OptionType");
		f[1] = Schema.createField(DataType.DOUBLE, "Price");
		f[2] = Schema.createField(DataType.DOUBLE, "Delta");
		f[3] = Schema.createField(DataType.DOUBLE, "Lambda");
		f[4] = Schema.createField(DataType.DOUBLE, "Rho");
		f[5] = Schema.createField(DataType.DOUBLE, "Theta");
		f[6] = Schema.createField(DataType.DOUBLE, "Gamma");
		f[7] = Schema.createField(DataType.DOUBLE, "Vega");	
		
		// Set Descriptions
		f[0].setDescription("The name of the type of option");
		f[1].setDescription("The price of the option");
		f[2].setDescription("The sensitivity in option value to changes in the underlying asset price");
		f[3].setDescription("The percent change in option value when the underlying asset price changes by one percent change");
		f[4].setDescription("The sensitivity in option value to changes in the interest rate");
		f[5].setDescription("The sensitivity in option value to changes in time to maturity");
		f[6].setDescription("The sensitivity in option value to changes in the options delta (or second derivative of asset price)");
		f[7].setDescription("The sensitivity in option value to changes in the volatility of the underlying asset");
		
		Schema schema = new Schema(null, f);
		Tuple tuple = schema.createTuple();
		int counter = 0;
		tuple.setString(counter++,type);
		for (Double element : info){
			tuple.setDouble(counter++, element);
		}
		return tuple;
	}

	public static CompleteDataType blackScholesTupleCustomFunctionResolver(CompleteDataType type, CompleteDataType option) {
		if (type.getDataType() != DataType.STRING && option.getElementType() != CompleteDataType.forDouble())
			return null;	
		Field[] f = new Field[8];
		f[0] = Schema.createField(DataType.STRING, "OptionType");
		f[1] = Schema.createField(DataType.DOUBLE, "Price");
		f[2] = Schema.createField(DataType.DOUBLE, "Delta");
		f[3] = Schema.createField(DataType.DOUBLE, "Lambda");
		f[4] = Schema.createField(DataType.DOUBLE, "Rho");
		f[5] = Schema.createField(DataType.DOUBLE, "Theta");
		f[6] = Schema.createField(DataType.DOUBLE, "Gamma");
		f[7] = Schema.createField(DataType.DOUBLE, "Vega");		
		f[0].setDescription("The name of the type of option");
		f[1].setDescription("The price of the option");
		f[2].setDescription("The sensitivity in option value to changes in the underlying asset price");
		f[3].setDescription("The percent change in option value when the underlying asset price changes by one percent change");
		f[4].setDescription("The sensitivity in option value to changes in the interest rate");
		f[5].setDescription("The sensitivity in option value to changes in time to maturity");
		f[6].setDescription("The sensitivity in option value to changes in the options delta (or second derivative of asset price)");
		f[7].setDescription("The sensitivity in option value to changes in the volatility of the underlying asset");
		Schema schema = new Schema(null, f);
		return CompleteDataType.forTuple(schema);		
	}

}
